@@ -101,7 +101,7 @@ use std::path::PathBuf;
 use structopt::StructOpt;
 use wasmtime::{Config, ProfilingStrategy};
 #[cfg(feature = "pooling-allocator")]
-use wasmtime::{InstanceLimits, ModuleLimits, PoolingAllocationStrategy};
+use wasmtime::{InstanceLimits, PoolingAllocationStrategy};
 
 fn pick_profiling_strategy(jitdump: bool, vtune: bool) -> Result<ProfilingStrategy> {
     Ok(match (jitdump, vtune) {
@@ -234,9 +234,8 @@ struct CommonOptions {
     #[structopt(long)]
     enable_cranelift_nan_canonicalization: bool,
 
-    /// Executing wasm code will consume fuel, limiting its execution.
     #[structopt(long)]
-    consume_fuel: bool,
+    fuel: Option<u64>,
 
     /// Executing wasm code will yield when a global epoch counter
     /// changes, allowing for async operation without blocking the
@@ -253,6 +252,12 @@ struct CommonOptions {
     #[structopt(long)]
     paged_memory_initialization: bool,
 
+    /// Disables the default of attempting to initialize linear memory via a
+    /// copy-on-write mapping.
+    #[cfg(feature = "memory-init-cow")]
+    #[structopt(long)]
+    disable_memory_init_cow: bool,
+
     /// Enables the pooling allocator, in place of the on-demand
     /// allocator.
     #[cfg(feature = "pooling-allocator")]
@@ -328,23 +333,23 @@ impl CommonOptions {
             config.dynamic_memory_guard_size(size);
         }
 
-        config.consume_fuel(self.consume_fuel);
+        // If fuel has been configured, set the `consume fuel` flag on the config.
+        if self.fuel.is_some() {
+            config.consume_fuel(true);
+        }
+
         config.epoch_interruption(self.epoch_interruption);
         config.generate_address_map(!self.disable_address_map);
         config.paged_memory_initialization(self.paged_memory_initialization);
+        #[cfg(feature = "memory-init-cow")]
+        config.memory_init_cow(!self.disable_memory_init_cow);
 
         #[cfg(feature = "pooling-allocator")]
         {
             if self.pooling_allocator {
-                let mut module_limits = ModuleLimits::default();
-                module_limits.functions = 50000;
-                module_limits.types = 10000;
-                module_limits.globals = 1000;
-                module_limits.memory_pages = 2048;
                 let instance_limits = InstanceLimits::default();
                 config.allocation_strategy(wasmtime::InstanceAllocationStrategy::Pooling {
                     strategy: PoolingAllocationStrategy::NextAvailable,
-                    module_limits,
                     instance_limits,
                 });
             }
