@@ -7,12 +7,12 @@ fn successful_instantiation() -> Result<()> {
     let mut config = Config::new();
     config.allocation_strategy(InstanceAllocationStrategy::Pooling {
         strategy: PoolingAllocationStrategy::NextAvailable,
-        module_limits: ModuleLimits {
+        instance_limits: InstanceLimits {
+            count: 1,
             memory_pages: 1,
             table_elements: 10,
             ..Default::default()
         },
-        instance_limits: InstanceLimits { count: 1 },
     });
     config.dynamic_memory_guard_size(0);
     config.static_memory_guard_size(0);
@@ -33,25 +33,36 @@ fn memory_limit() -> Result<()> {
     let mut config = Config::new();
     config.allocation_strategy(InstanceAllocationStrategy::Pooling {
         strategy: PoolingAllocationStrategy::NextAvailable,
-        module_limits: ModuleLimits {
+        instance_limits: InstanceLimits {
+            count: 1,
             memory_pages: 3,
             table_elements: 10,
             ..Default::default()
         },
-        instance_limits: InstanceLimits { count: 1 },
     });
     config.dynamic_memory_guard_size(0);
     config.static_memory_guard_size(65536);
     config.static_memory_maximum_size(3 * 65536);
+    config.wasm_multi_memory(true);
 
     let engine = Engine::new(&config)?;
 
-    // Module should fail to validate because the minimum is greater than the configured limit
-    match Module::new(&engine, r#"(module (memory 4))"#) {
-        Ok(_) => panic!("module compilation should fail"),
+    // Module should fail to instantiate because it has too many memories
+    match Module::new(&engine, r#"(module (memory 1) (memory 1))"#) {
+        Ok(_) => panic!("module instantiation should fail"),
         Err(e) => assert_eq!(
             e.to_string(),
-            "memory index 0 has a minimum page size of 4 which exceeds the limit of 3"
+            "defined memories count of 2 exceeds the limit of 1",
+        ),
+    }
+
+    // Module should fail to instantiate because the minimum is greater than
+    // the configured limit
+    match Module::new(&engine, r#"(module (memory 4))"#) {
+        Ok(_) => panic!("module instantiation should fail"),
+        Err(e) => assert_eq!(
+            e.to_string(),
+            "memory index 0 has a minimum page size of 4 which exceeds the limit of 3",
         ),
     }
 
@@ -101,13 +112,10 @@ fn memory_init() -> Result<()> {
     let mut config = Config::new();
     config.allocation_strategy(InstanceAllocationStrategy::Pooling {
         strategy: PoolingAllocationStrategy::NextAvailable,
-        module_limits: ModuleLimits {
-            memory_pages: 2,
-            table_elements: 0,
-            ..Default::default()
-        },
         instance_limits: InstanceLimits {
             count: 1,
+            memory_pages: 2,
+            table_elements: 0,
             ..Default::default()
         },
     });
@@ -137,13 +145,10 @@ fn memory_guard_page_trap() -> Result<()> {
     let mut config = Config::new();
     config.allocation_strategy(InstanceAllocationStrategy::Pooling {
         strategy: PoolingAllocationStrategy::NextAvailable,
-        module_limits: ModuleLimits {
-            memory_pages: 2,
-            table_elements: 0,
-            ..Default::default()
-        },
         instance_limits: InstanceLimits {
             count: 1,
+            memory_pages: 2,
+            table_elements: 0,
             ..Default::default()
         },
     });
@@ -196,12 +201,12 @@ fn memory_zeroed() -> Result<()> {
     let mut config = Config::new();
     config.allocation_strategy(InstanceAllocationStrategy::Pooling {
         strategy: PoolingAllocationStrategy::NextAvailable,
-        module_limits: ModuleLimits {
+        instance_limits: InstanceLimits {
+            count: 1,
             memory_pages: 1,
             table_elements: 0,
             ..Default::default()
         },
-        instance_limits: InstanceLimits { count: 1 },
     });
     config.dynamic_memory_guard_size(0);
     config.static_memory_guard_size(0);
@@ -239,12 +244,12 @@ fn table_limit() -> Result<()> {
     let mut config = Config::new();
     config.allocation_strategy(InstanceAllocationStrategy::Pooling {
         strategy: PoolingAllocationStrategy::NextAvailable,
-        module_limits: ModuleLimits {
+        instance_limits: InstanceLimits {
+            count: 1,
             memory_pages: 1,
             table_elements: TABLE_ELEMENTS,
             ..Default::default()
         },
-        instance_limits: InstanceLimits { count: 1 },
     });
     config.dynamic_memory_guard_size(0);
     config.static_memory_guard_size(0);
@@ -252,12 +257,22 @@ fn table_limit() -> Result<()> {
 
     let engine = Engine::new(&config)?;
 
-    // Module should fail to validate because the minimum is greater than the configured limit
+    // Module should fail to instantiate because it has too many tables
+    match Module::new(&engine, r#"(module (table 1 funcref) (table 1 funcref))"#) {
+        Ok(_) => panic!("module compilation should fail"),
+        Err(e) => assert_eq!(
+            e.to_string(),
+            "defined tables count of 2 exceeds the limit of 1",
+        ),
+    }
+
+    // Module should fail to instantiate because the minimum is greater than
+    // the configured limit
     match Module::new(&engine, r#"(module (table 31 funcref))"#) {
         Ok(_) => panic!("module compilation should fail"),
         Err(e) => assert_eq!(
             e.to_string(),
-            "table index 0 has a minimum element size of 31 which exceeds the limit of 10"
+            "table index 0 has a minimum element size of 31 which exceeds the limit of 10",
         ),
     }
 
@@ -316,13 +331,10 @@ fn table_init() -> Result<()> {
     let mut config = Config::new();
     config.allocation_strategy(InstanceAllocationStrategy::Pooling {
         strategy: PoolingAllocationStrategy::NextAvailable,
-        module_limits: ModuleLimits {
-            memory_pages: 0,
-            table_elements: 6,
-            ..Default::default()
-        },
         instance_limits: InstanceLimits {
             count: 1,
+            memory_pages: 0,
+            table_elements: 6,
             ..Default::default()
         },
     });
@@ -369,12 +381,12 @@ fn table_zeroed() -> Result<()> {
     let mut config = Config::new();
     config.allocation_strategy(InstanceAllocationStrategy::Pooling {
         strategy: PoolingAllocationStrategy::NextAvailable,
-        module_limits: ModuleLimits {
+        instance_limits: InstanceLimits {
+            count: 1,
             memory_pages: 1,
             table_elements: 10,
             ..Default::default()
         },
-        instance_limits: InstanceLimits { count: 1 },
     });
     config.dynamic_memory_guard_size(0);
     config.static_memory_guard_size(0);
@@ -413,14 +425,12 @@ fn instantiation_limit() -> Result<()> {
     let mut config = Config::new();
     config.allocation_strategy(InstanceAllocationStrategy::Pooling {
         strategy: PoolingAllocationStrategy::NextAvailable,
-        module_limits: ModuleLimits {
+        instance_limits: InstanceLimits {
+            count: INSTANCE_LIMIT,
             memory_pages: 1,
             table_elements: 10,
             ..Default::default()
         },
-        instance_limits: InstanceLimits {
-            count: INSTANCE_LIMIT,
-        },
     });
     config.dynamic_memory_guard_size(0);
     config.static_memory_guard_size(0);
@@ -465,12 +475,12 @@ fn preserve_data_segments() -> Result<()> {
     let mut config = Config::new();
     config.allocation_strategy(InstanceAllocationStrategy::Pooling {
         strategy: PoolingAllocationStrategy::NextAvailable,
-        module_limits: ModuleLimits {
+        instance_limits: InstanceLimits {
+            count: 2,
             memory_pages: 1,
             table_elements: 10,
             ..Default::default()
         },
-        instance_limits: InstanceLimits { count: 2 },
     });
     let engine = Engine::new(&config)?;
     let m = Module::new(
@@ -512,6 +522,42 @@ fn preserve_data_segments() -> Result<()> {
     Ok(())
 }
 
+#[test]
+fn multi_memory_with_imported_memories() -> Result<()> {
+    // This test checks that the base address for the defined memory is correct for the instance
+    // despite the presence of an imported memory.
+
+    let mut config = Config::new();
+    config.allocation_strategy(InstanceAllocationStrategy::Pooling {
+        strategy: PoolingAllocationStrategy::NextAvailable,
+        instance_limits: InstanceLimits {
+            count: 1,
+            memories: 2,
+            memory_pages: 1,
+            ..Default::default()
+        },
+    });
+    config.wasm_multi_memory(true);
+
+    let engine = Engine::new(&config)?;
+    let module = Module::new(
+        &engine,
+        r#"(module (import "" "m1" (memory 0)) (memory (export "m2") 1))"#,
+    )?;
+
+    let mut store = Store::new(&engine, ());
+
+    let m1 = Memory::new(&mut store, MemoryType::new(0, None))?;
+    let instance = Instance::new(&mut store, &module, &[m1.into()])?;
+
+    let m2 = instance.get_memory(&mut store, "m2").unwrap();
+
+    m2.data_mut(&mut store)[0] = 0x42;
+    assert_eq!(m2.data(&store)[0], 0x42);
+
+    Ok(())
+}
+
 #[test]
 fn drop_externref_global_during_module_init() -> Result<()> {
     struct Limiter;
@@ -530,8 +576,10 @@ fn drop_externref_global_during_module_init() -> Result<()> {
     config.wasm_reference_types(true);
     config.allocation_strategy(InstanceAllocationStrategy::Pooling {
         strategy: PoolingAllocationStrategy::NextAvailable,
-        module_limits: Default::default(),
-        instance_limits: InstanceLimits { count: 1 },
+        instance_limits: InstanceLimits {
+            count: 1,
+            ..Default::default()
+        },
     });
 
     let engine = Engine::new(&config)?;
@@ -569,3 +617,50 @@ fn drop_externref_global_during_module_init() -> Result<()> {
 
     Ok(())
 }
+
+#[test]
+#[cfg(target_pointer_width = "64")]
+fn instance_too_large() -> Result<()> {
+    let mut config = Config::new();
+    config.allocation_strategy(InstanceAllocationStrategy::Pooling {
+        strategy: PoolingAllocationStrategy::NextAvailable,
+        instance_limits: InstanceLimits {
+            size: 16,
+            count: 1,
+            ..Default::default()
+        },
+    });
+
+    let engine = Engine::new(&config)?;
+    let expected = "\
+instance allocation for this module requires 304 bytes which exceeds the \
+configured maximum of 16 bytes; breakdown of allocation requirement:
+
+ * 78.95% - 240 bytes - instance state management
+ * 5.26% - 16 bytes - jit store state
+";
+    match Module::new(&engine, "(module)") {
+        Ok(_) => panic!("should have failed to compile"),
+        Err(e) => assert_eq!(e.to_string(), expected),
+    }
+
+    let mut lots_of_globals = format!("(module");
+    for _ in 0..100 {
+        lots_of_globals.push_str("(global i32 i32.const 0)\n");
+    }
+    lots_of_globals.push_str(")");
+
+    let expected = "\
+instance allocation for this module requires 1904 bytes which exceeds the \
+configured maximum of 16 bytes; breakdown of allocation requirement:
+
+ * 12.61% - 240 bytes - instance state management
+ * 84.03% - 1600 bytes - defined globals
+";
+    match Module::new(&engine, &lots_of_globals) {
+        Ok(_) => panic!("should have failed to compile"),
+        Err(e) => assert_eq!(e.to_string(), expected),
+    }
+
+    Ok(())
+}
